/*
HTML.h
*/

#ifndef __HTML_H
#define __HTML_H

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include "Config.h"
#include "Utils.h"
#include "XSocket.h"
#include "XVector.h"
#include "XString.h"
#include "Language.h"
#include "UserOptions.h"
#include "Addressbook.h"
#include "ClientNNTP.h"
#include "JavaScripts.h"

class HTML
  {
  private:
    FILE *fout;
    int iout;
    long MAXMSGSFORPAGE;
    XSocket *OneSocket;
    XString XBuffer;
    void PrintStatusLine (const char *user, UserOptions *uo, bool displayClock);
  public:
    HTML (FILE *FR);
    HTML (int FR);
    HTML (XSocket *aXS);
    virtual ~HTML (void);
    void setMAXMSGSFORPAGE (long maxmsgs);
    void WRITE (const char *fmt, ...);    
    void pcWRITE (const char *pc);
    void pcWRITEBuffer (const char *pc, xulong len);
    void printCookie (const char *cookieline);
    void Head (const char *contenttype, const char *name, const char *filename);
    void HeadInline (const char *contenttype, const char *filename, xulong len);
    void HTMLBegin (void);
    void Title (const char *title, const char *bgcolor, const char *txtcolor, const char *img_bg, bool include_js, int refresh_time, const char *newurl);
    void HTMLEnd (const char *bgtablecopyrightcolor);
    void EmulatePageNotFound (void);    
    void BeginTable (int border, int cellpadding, int cellspacing, int widthpercent);
    void EndTable (void);
    void BeginTR (void);
    void BeginTR (const char *bgcolor);
    void EndTR (void);
    void BeginTD (void);
    void BeginTDCenter (void);
    void BeginTDColSpanCenter (int acolspan);
    void BeginTDColSpan (int acolspan);    
    void BeginTD (const char *bgcolor);
    void BeginTD (const char *bgcolor, const char *align);            
    void BeginTD (const char *bgcolor, const char *align, int colspan);    
    void BeginTD (int widthpercent);
    void BeginTD (int widthpercent, const char *facefont, int sizefont, const char *align);    
    void BeginTD (bool nowrap, int widthpercent, int rowspan, const char *facefont, int sizefont, const char *align);
    void EndTD (void);
    void EmptyCellx1 (void);
    void EmptyCellx2 (void);
    void BeginFont (const char *face, int size, const char *color);    
    void EndFont (void);
    void BeginBold (void);
    void EndBold (void);
    void BeginItalic (void);
    void EndItalic (void);
    void BeginUnderline (void);
    void EndUnderline (void);
    void BR (void);
    void Small (const char *linea);
    void Rectangle (const char *linea);
    void RedRectangle (const char *linea);    
    void BeginNOBR (void);
    void EndNOBR(void);
    void JavaScript (const char *codigo);
    void H1 (const char *linea);
    void H2 (const char *linea);
    void H3 (const char *linea);
    void H4 (const char *linea);
    void H5 (const char *linea);
    void Label (const char *linea);
    void RedLabel (const char *linea);
    void GreenLabel (const char *linea);
    void URL (const char *caption, const char *url);
    void Image (const char *urlimage, const char *label);
    void URLIMAGE (const char *label, const char *url, const char *urlimage);
    void PrintString (const char *linea);
    void pr (const char *linea);    
    void Redirect (const char *newurl);
    void Linea (void);
    void Error (const char *error);
    void Msg (const char *msg);
    void ErrorPage (const char *error);
    void ErrorPageBack (const char *stlang, const char *user, const char *connid, UserOptions *uo, const char *error, int way);
    void MsgPage (const char *msg, const char *lang);    
    void soErrorPage (const char *error);
    void OkPage (const char *stlang, const char *user, const char *connid, UserOptions *uo, const char *okmsg);
    void Ok (const char *linea);
    void BeginPre (void);
    void EndPre (void);
    void BeginCenter (void);
    void EndCenter (void);
    void BeginFormPost (const char *name, const char *action);
    void BeginFormPostMultipartFormData (const char *name, const char *action);
    void BeginFormGet (const char *name, const char *action);
    void EndForm (void);
    void FormHiddenQuoted (const char *name, const char *value);    
    void FormHidden (const char *name, const char *value);
    void FormHidden (const char *name, int value);    
    void FormEdit (int max, int size, const char *name, const char *value);
    void FormFile (int max, int size, const char *name, const char *value, const char *accept);
    void FormTextArea (int rows, int cols, const char *name, const char *value, bool readonly);    
    void FormPassword (int max, int size, const char *name, const char *value);
    void FormCheckbox (const char *name, const char *value, bool checked);
    void JSFormCheckbox (const char *name, const char *value, const char *onclick, bool checked);
    void FormImage (const char *name, const char *src);
    void FormButton (const char *name, const char *value);
    void FormReset (const char *value);
    void FormBeginSelect (const char *formname, const char *selectname, const char *submitbutton, bool use_javascript);
    void FormBeginMultipleSelect (const char *formname, const char *selectname, const char *submitbutton, bool use_javascript);    
    void FormAddOptionSelect (bool selected, const char *value, const char *label);
    void FormEndSelect ();
    void dumpSL (StringList *SL, bool PutNewLines);
    void dumpPChar (const char *pc);    
    void dumpFile (TBuffer fn, TBuffer ContentType);
    void BotoneraDisplayMsg (const char *lang, const char *user, const char *connid, UserOptions *uo, bool isdeleted, 
                             ULongList *LL, bool showreplyall, bool isTopBar);
    void BotoneraMailboxIndex (const char *lang, const char *user, const char *connid, UserOptions *uo);
    void BotoneraComun (int cmdactual, const char *lang, const char *user, const char *connid, UserOptions *uo);
    void BotoneraAttachs (const char *lang, const char *user, const char *connid, UserOptions *uo);
    void BotoneraComposeMsg (int thecmd, const char *lang, const char *user, const char *connid, UserOptions *uo);
    void LineaLanguage (void);
    void PrintIndexPage (const char *lang, const char *mailbox, const char *user, const char *connid, 
                         StringList *SLHeaders, StringList *SLmailboxes, UserOptions *uo);
    void PrintMessageHeader 
                      (const char *lang, const char *user, const char *connid, 
                       long nummsgs, const char *aflags, const char *ato, const char *afrom, const char *areplyto, const char *acc, 
                       const char *asubject, const char *adate, const char *asize,
                       StringList *SLAttach, UserOptions *uo, ULongList *LL);
    void PrintMessageTail 
                      (const char *lang, const char *user, const char *connid, 
                       long nummsgs, const char *aflags, const char *ato, const char *afrom, const char *areplyto, const char *acc, 
                       const char *asubject, const char *adate, const char *asize,
                       StringList *SLAttach, UserOptions *uo, ULongList *LL);                                              
    void dumpAttach (const char *chunk, xulong len, StringList *SLAttachInfo);
    void dumpRawAttachPC (const char *pcheader, xulong lenheader, const char *pcbody, xulong lenbody);
    void dumpRawAttachSL (StringList *SLAttachHeader, const char *pcbody, xulong lenbody);
    void dumpAttachWithoutHead (const char *chunk, xulong len);
    void PrintConfirmPage (const char *stcmd, const char *stlang, const char *mailbox, const char *user, const char *connid, int validator, int pageid, UserOptions *uo);
    void PrintMailboxesPage (const char *lang, const char *mailbox, const char *user, const char *connid, 
                             StringList *SLMailboxes, UserOptions *uo);    
    void PrintComposeMsgPage (int thecmd, int subcmd, const char *lang, const char *user, const char *connid, 
                              const char *to, const char *cc, const char *bcc, const char *subject, 
                              const char *replyto, const char *from, const char *date,
                              const char *firsttext,
                              UserOptions *uo);
    void PrintShowSignature (const char *stlang, const char *user, const char *connid, long actualpos, 
                             const char *signature, UserOptions *uo);
    void PrintShowOptions (const char *stlang, const char *user, const char *connid, UserOptions *uo);
    void PrintInvalidPage (const char *stlang, const char *user, const char *connid, UserOptions *uo);
    void PrintBadCookiePage (const char *stlang, const char *user, const char *connid, UserOptions *uo);    
    void PrintShowAttachs (const char *stlang, const char *user, const char *connid, UserOptions *uo);
    void PrintAddBook_Show (int thecmd, const char *stlang, const char *user, const char *connid, UserOptions *uo, Addressbook *ab, int sortby);
    void PrintAddBook_Dump (int thecmd, const char *stlang, const char *user, const char *connid, UserOptions *uo, Addressbook *ab, int sortby);    
    void PrintAddBook_EditEntry (int thecmd, const char *stlang, const char *user, const char *connid, UserOptions *uo, Addressbook *ab, int numentry);
    void PrintDebugPage (const char *stlang, const char *user, const char *connid, UserOptions *uo);
    void PrintFullHeaders (StringList *SLHeaders, UserOptions *uo, const char *asize);
    void PrintTCPTimeoutPage (const char *lang);
    void PrintShowNNTPManageGroups (const char *stlang, const char *user, const char *connid, UserOptions *uo, ClientNNTP *cn, const char *text2search);
    };
  
#endif  
  
