/*
Utils.h
*/

#ifndef __UTILS_H
#define __UTILS_H

#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <limits.h>
#include <stdio.h>
#include <time.h>
#include <unistd.h>
#include <pwd.h>
#include <malloc.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <ctype.h>
#include <syslog.h>
#include <dirent.h>
#include <sys/times.h>                                             
#include <signal.h>

#include "Config.h"
#include "XSocket.h"

#if defined (LINUX)
#include <asm/errno.h>
#elif defined (HP) 
#include <sys/errno.h>
#elif defined (SUN) 
#include <sys/errno.h>
#endif

#include "XString.h"
#include "XVector.h"
typedef XVector<XString> StringList;
typedef XVector<xulong> ULongList;

#define STR2(X)     #X
#define STR(X)      STR2(X)

#define CMD_CONF_ACT          -3    //SPECIALS 
#define CMD_INVALIDPAGE       -2
#define CMD_INVALID           -1
#define CMD_HOME               0    //MISC
#define CMD_LOGIN              1
#define CMD_LOGOUT             2
#define CMD_HELP               3
#define CMD_MG_DUMPFULLMSG     4    //MESSAGE
#define CMD_MG_DISPLAY         5
#define CMD_MG_COPY            6
#define CMD_MG_MOVE            7                           
#define CMD_MG_DELDISPLAY      8
#define CMD_MG_UNDELDISPLAY    9
#define CMD_MB_INDEX          10    //MAILBOX
#define CMD_MB_SHOW           11
#define CMD_MB_CHANGE         12
#define CMD_MB_CREATE         13
#define CMD_MB_DELETE         14
#define CMD_MB_RENAME         15
#define CMD_MB_EXPUNGE        16
#define CMD_FG_DELETE         17    //FLAGS
#define CMD_FG_UNDELETE       18
#define CMD_FG_ANSWERED       19
#define CMD_FG_UNANSWERED     20
#define CMD_FG_FLAGGED        21                           
#define CMD_FG_UNFLAGGED      22
#define CMD_FG_SEEN           23
#define CMD_FG_UNSEEN         24
#define CMD_FG_RECENT         25
#define CMD_FG_UNRECENT       26    
#define CMD_CM                27    //COMPOSE MESSAGE   
#define CMD_CMNNTP            28
#define CMD_CM_SEND           29
#define CMD_CM_SENDNNTP       30
#define CMD_CM_CANCEL         31
#define CMD_CM_REPLY          32
#define CMD_CM_REPLYALL       33
#define CMD_CM_REPLYGROUP     34
#define CMD_CM_FORWARD        35
#define CMD_CM_CLEANALL       36
#define CMD_CM_SAVE           37
#define CMD_OP                38    //OPTIONS
#define CMD_OP_SAVE           39
#define CMD_OP_CANCEL         40
#define CMD_AT_DUMP           41    //ATTACHMENTS
#define CMD_AT_SHOW           42
#define CMD_AT_SAVE           43
#define CMD_AT_ERASE          44
#define CMD_AB                45    //ADDRESSBOOK
#define CMD_AB_SAVEENTRY      46
#define CMD_AB_EDITENTRY      47
#define CMD_AB_SHOWALL        48
#define CMD_AB_FROM_CM        49
#define CMD_AB_DELEENTRIES    50
#define CMD_AB_NEWENTRY       51
#define CMD_AB_SORT           52
#define CMD_AB_TAKEADDRESS    53
#define CMD_MB_DUMP           54
#define CMD_AB_DUMP           55
#define CMD_CHECK_COOKIE      56
#define CMD_NOOP              57
#define CMD_DEBUG             58
#define CMD_MB_SEARCH         59
#define CMD_MB_SORT           60
#define CMD_MG_FG_DELETE      61   
#define CMD_MG_FG_UNDELETE    62   
#define CMD_MG_FG_ANSWERED    63   
#define CMD_MG_FG_UNANSWERED  64  
#define CMD_MG_FG_FLAGGED     65                             
#define CMD_MG_FG_UNFLAGGED   66  
#define CMD_MG_FG_SEEN        67  
#define CMD_MG_FG_UNSEEN      68  
#define CMD_MG_FG_SEARCHED    69  
#define CMD_MG_FG_UNSEARCHED  70  
#define CMD_MG_DUMPFULLHEADER 71
#define CMD_AB_ADDFIELDS      72
#define CMD_OP_NNTP             73
#define CMD_OP_NNTP_CANCEL      74
#define CMD_OP_NNTP_SAVE        75
#define CMD_OP_NNTP_SEARCH      76
#define CMD_OP_NNTP_ADDSELECTED 77
#define CMD_BD_UUDECODE       78
#define CMD_NW_LIST_DUMP      79

#define CMD_HELP_LOGIN          0
#define CMD_HELP_INDEXMAILBOX   1
#define CMD_HELP_DISPLAYMSG     2
#define CMD_HELP_SHOWMAILBOXES  3
#define CMD_HELP_COMPOSEMSG     4
#define CMD_HELP_SHOWOPTIONS    5
#define CMD_HELP_ADDBOOK        6
#define CMD_HELP_SHOWATTACHS    7
#define CMD_HELP_ABOUT          8
#define CMD_HELP_AB_EDITENTRY   9
#define CMD_HELP_AB_NEWENTRY   10

#define SP ' '
#define TAB '\t'

#define HTML_CR           "\n"
#define HTML_SP           "&#160;"
#define HTML_MENOR        "&#60;"
#define HTML_MAYOR        "&#62;"
#define HTML_DCOMILLAS    "&#34;"
#define HTML_HSP          "&nbsp;"
#define HTML_PORC         "&#37;"
#define HTML_SOSTENIDO    "&#35;"
#define HTML_SQU_OPEN_PAR "&#91;"
#define HTML_SQU_CLOS_PAR "&#93;"

#define HTML_nTILDE       "&ntilde;"  //241
#define HTML_NTILDE       "&Ntilde;"  //209

#define HTML_aACUTE       "&aacute;"  //225
#define HTML_aGRAVE       "&agrave;"  //224

#define HTML_eACUTE       "&eacute;"  //233
#define HTML_eGRAVE       "&egrave;"  //232

#define HTML_iACUTE       "&iacute;"  //237
#define HTML_iGRAVE       "&igrave;"  //236

#define HTML_oACUTE       "&oacute;"  //243 
#define HTML_oGRAVE       "&ograve;"  //242

#define HTML_uACUTE       "&uacute;"  //250 
#define HTML_uGRAVE       "&ugrave;"  //249

#define HTML_AACUTE       "&Aacute;"  //193
#define HTML_AGRAVE       "&Agrave;"  //192

#define HTML_EACUTE       "&Eacute;"  //201
#define HTML_EGRAVE       "&Egrave;"  //200

#define HTML_IACUTE       "&Iacute;"  //205
#define HTML_IGRAVE       "&Igrave;"  //204

#define HTML_OACUTE       "&Oacute;"  //211
#define HTML_OGRAVE       "&Ograve;"  //210

#define HTML_UACUTE       "&Uacute;"  //218
#define HTML_UGRAVE       "&Ugrave;"  //217


#define FORM_SELECT_SEPARATOR "----------"

#define IMAP_ERROR_FATAL (long) 1000

#define INBOX HTML_DCOMILLAS"INBOX"HTML_DCOMILLAS

//#define HTML_VERSION "3.2"
//#define HTML_DOCTYPE "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML "HTML_VERSION" Final//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">"
#define HTML_VERSION "4.01"
#define HTML_DOCTYPE "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML "HTML_VERSION" Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">"

#define WHITE      "#FFFFFF"
#define BLACK      "#000000"
#define RED        "#FF0000"
#define GREEN      "#00AA00"
#define BLUE       "#0000FF"
#define BLUE2      "#008080"

#define NO_SORT    1000
#define SORTTHREAD 1001

//FLAGS: only one char!
#define FRECENT      'R'
#define FNEW         'N' 
#define FUNSEEN      'U'
#define FFLAGGED     'F'
#define FANSWERED    'A'
#define FDELETED     'D'
#define FSEARCHED    '@'
#define FEMPTY       '-'

#define POS_FLAG_DELETED   0
#define POS_FLAG_ANSWERED  1
#define POS_FLAG_NEW       2
#define POS_FLAG_IMPORTANT 3
#define POS_FLAG_SEARCHED  4

#define CMAXHBUFFER 20000
typedef char THBuffer[CMAXHBUFFER + 1];
#define CMAXBUFFER 2000
typedef char TBuffer[CMAXBUFFER + 1];
#define CSMALLBUFFER 200
typedef char TSBuffer[CSMALLBUFFER + 1];
#define CMAXFILENAME 255
typedef char TFileName[CMAXFILENAME + 1];
#define MAXLENGTHLINE 500
typedef char LINEA [MAXLENGTHLINE + 1];
#define CMAXIPNAME 50
typedef char IPNAME [CMAXIPNAME + 1];
#define CMAXNUMBER 20
typedef char TNumber [CMAXNUMBER + 1];

#define MAXLENGTHSKIN 50
typedef int FONTSIZE;
#define MAXLENGTHFONTNAME 50
typedef char FONTNAME [MAXLENGTHFONTNAME + 1];
#define MAXLENGTHCOLORHTML 7
typedef char COLORHTML [MAXLENGTHCOLORHTML + 1];

#define MAX_FILENAME_SIZE 1024
char *get_filename(const char *str);

void initStr (char *thestr);
void initStr (char *thestr, const char *consstr);
char *Cut (const char *pc, int maxlong, TBuffer abuf);
bool CreateDir (char *path, mode_t mode);
bool DeleteFile (const char *fname);
int CreateAndLockFile (char *fn);
void EscribePIDenLockFile (int fd);
void UnlockAndCloseFile (int fd);
int IsLocked (char *fn, int *fd);
int GetPIDFromLockFile (int fd);
long GetTimeFromLockFileOpening (char *fn);
long GetTimeFromLockFile (int fd);
void SaveFileDebug (char *fn, char *pc, int len);
void SaveFileDebug (char *fn, char *pc);
char *ReturnUsersDir (const char *basepath, const char *user, const char *server, TBuffer ABuf);
char *ReturnUserLockFileName (const char *user, TBuffer ABuf);
char *ReturnUserAttachsDir (const char *basepath, const char *user, const char *server, TBuffer ABuf);
char *ReturnSignatureFileName (const char *basepath, const char *user, const char *server, TBuffer ABuf);
char *ReturnSubscribedNNGroupsFileName (const char *basepath, const char *user, const char *server, TBuffer ABuf);
char *ReturnRcNNGroupsFileName (const char *basepath, const char *user, const char *server, TBuffer ABuf);
char *ReturnAllNNGroupsFileName (const char *server, TBuffer abuf);
char *ReturnUserOptionsFileName (const char *basepath, const char *user, const char *server, TBuffer ABuf);
char *ReturnUserAddressbookFileName (const char *basepath, const char *user, const char *server, TBuffer ABuf);
char *ReturnUserSavedMsgFileName (const char *basepath, const char *user, const char *server, TBuffer ABuf);
char *ReturnUserSavedNNTPMsgFileName (const char *basepath, const char *user, const char *server, TBuffer ABuf);
void QuitaRetornoCarroDeLinea (char *linea);
char *xstrdup (char const *str);
void Randomize ();
int Rand (int max);
char *MakeTmpName (TBuffer fntmp, int maxlong);
char *getExtensionFromFile (const char *fn);
bool FileExists (const char *fn);
int FileSize (const char *fn);
time_t FileDate (char *fn);
char *xucase (char *s);
char *xlcase (char *s);
int TranslateCmd (const char *stcmd);
int TranslateHelpCmd (const char *sthelp);
char *xitoa (int n, TNumber abuf);
char *xltoa (long n, TNumber abuf);
int xatoidef (const char *stnum, int def);
long int xatoldef (const char *stnum, long int def);
char *make_session_id (const char *server, TBuffer abuf);
char *extract_server_from_connid (const char *connid, TBuffer Buf);
char *make_root_socket (TBuffer Buf);
char *make_session_socket (char *username, char *connid, TBuffer Buf);
void ini (char *connid, char *stlang, char *stcmd, char *allparms, char *allparms2,
          char *parm1, char *parm2, char *parm3, char *parm4, char *parm5);
int NCstrstr (const char *linea, const char *token);
int Cstrstr (const char *linea, const char *token);
void OPENLOG (void);
void CLOSELOG (void);
void LOG (char *fmt, ...);
void LOG (long i);
void DEBUG (char *fmt, ...);
void DEBUG (int imsg);
char *FormatCountBytes (long count, TBuffer abuf);
char *FormatCountBytes1Dec (long count, TBuffer abuf);
void Redirect (const char *newurl);
void ErrorPage (const char *error);
void CGIErrorPageConnectionRefused (const char *lang);
void PrintHelpPage (char *topic);
bool CopyFile (char *oldfile, char *newfile);
bool CopyFileOverWrite (const char *oldfile, const char *newfile);
char *Text2HTML (const char *ast, char *res);
bool IsEmpty (const char *pc);
bool IsNull (const char *pc);
bool IsGoodValidator (int cmd, int validator, int validatorgood);
void Ltrim (char *linea);
void Rtrim (char *linea);
void trim  (char *linea);
void Rclean (char *linea);
const char *ForceSpace (const char *value);
char *CutSign (const char *pc, int maxlong, const char *signal, TBuffer abuf);
char *getLineFile (const char *flname, int x, int y, TBuffer abuf);
void CreaFileSavedMsg (const char *fn);
bool ConcatLineToLineOfFile (const char *fn, int numlinea, const char *whatadd, const char *sep, int MAX);
int CountChar (char *string, char tocount);
pid_t WritePidFile (void);
void DeletePidFile (void);
void DelAllFilesFromDir (char *thedir);
char *getMailcapTipoFromFilename (const char *fn, const char *fnmailcap);
void PrintHomePage (char *lang);
void DumpHTMLPage (char *fn);
long min (long i,long j);
void XTICK (char *label);
void PChar2SL (char *chunk, StringList *SL);
void ResaltaURLsEnText (char *txt, XSocket *aXS, int MaxLongLine, bool IsUencoded, TBuffer urltouudecode);
bool IsMultiplo (int dividendo, int divisor);
void xscanf (const char *linea, 
            char *c0,
            int *int1, int *int2, int *int3, 
            char *c1, char *c2, char *c3, char *c4, char *c5, char *c6, char *c7, char *c8);
char *ProcesaDate (char *tmp, TBuffer abuf);
int GetBufferFromFile (char *afile, char *abuf, int maxlength);
char *bool2char (bool b);
bool char2bool (char *c);
char *DisplayFlags (const char *flags, bool clickable, const char *molde, int show_way, TBuffer abuf);
int getNumChilds (void);
int SeparaTokensEnStringList (const char *pc, const char sep, StringList *SL);
int CountTokensEnString (const char *pc, const char sep);
void ExtraeAddresses (StringList *SL, char *linea, bool ClearSL);
void ExtraeFullAddresses (StringList *SL, const char *linea, bool ClearSL);
bool MustDoReplayAll (const char *ato, const char *afrom, const char *areplyto, const char *acc);
char *getReplyAllAddresses (const char *ato, const char *afrom, const char *areplyto, const char *acc);
int xvsnprintf(char *str, size_t size, const char *fmt, va_list ap);
int xsnprintf(char *str, size_t size, const char *fmt, ...);
void DisableHTMLTokens (char *htmltxt);
int MergeStringList (StringList *SLBase, StringList *SL2Merge, bool check_exists);
void MEMLOG (char *LineInfo);
bool StringsLike (const char* text, const char* wildcard);
bool SavePchar (const char *pc, int len, const char *fn);
                          
#endif
